#ifndef AIRFOIL_H
#define AIRFOIL_H

#include "spline.h"

/************************************************************************/
/*                                                                      */
/*         XX      XX W           W  III  NN    N    GGGG               */
/*           XX  XX    W         W    I   N N   N   G                   */
/*             XX       W   W   W     I   N  N  N   G  GGG              */
/*           XX  XX      W W W W      I   N   N N   G    G              */
/*         XX      XX     W   W      III  N    NN    GGGG               */
/*                                                                      */
/*  QtAirfoil Version 0.1        U.Lehnert                   3 /2010    */
/*                                                                      */
/*  Berechnung der Skelettlinie von Tragflügel-Profilen,                */
/*  deren aerodynamischen Kenngrößen                                    */
/*  und deren Unterteilung in Segmente zur Benutzung mit XWing          */
/*                                                                      */
/************************************************************************/


/*

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

class Airfoil;

class Camberline
{

public:

  Camberline(int NumberOfPanels,
             // leading edge and trailing edge panel density
             double LED,
             double TED);
  ~Camberline();
  
  // return number of points
  int npoints();
  
  double xpoint(int i);
  double ypoint(int i);
  
  // the x-position of the camber line panels are computed during initialization
  // the y-positions have to be set using one of these methode
  void sety(int i, double y);
  void computePoints(Airfoil *af);
  
  // compute the aerodynamic coefficients of a camberline
  // with the given paneling
  void computeCoeff(
    double *cl0,        // lift at zero angla of attack
    double *alfa0,      // angle of attack for zero lift in radian
    double *cm0,        // momentum
    double *dclda,      // lift slope in 1/radian
    double *asf,        // angle of attack for suction-free entrance
    double *clsf        // lift coefficient for suction-free entrance
  );
  
private:
  
  int npan;
  double *xs;
  double *ys;
  double *aoa;  // panel angle of attack
  
};

class Airfoil
{

public:

  Airfoil(int NumberOfPoints,
         double *x,
         double *y);
  ~Airfoil();

  Spline2D *outline;
  Spline *splx;
  Spline *sply;

  // return number of points
  int npoints();
  
  // return airfoil surface coordinate points
  double px(int i);
  double py(int i);

  // normalize airfoil
  // point with lowest x (on spline) -> (0, 0)
  // first and last points both having x=1
  // first and last points symmetrical to (1, 0)
  void normalize();

private:

  int np;
  double *xp;
  double *yp;

public:

};

#endif
