/************************************************************************/
/*                                                                      */
/*         XX      XX W           W  III  NN    N    GGGG               */
/*           XX  XX    W         W    I   N N   N   G                   */
/*             XX       W   W   W     I   N  N  N   G  GGG              */
/*           XX  XX      W W W W      I   N   N N   G    G              */
/*         XX      XX     W   W      III  N    NN    GGGG               */
/*                                                                      */
/*  QtAirfoil Version 0.1        U.Lehnert                   3 /2010    */
/*                                                                      */
/*  Berechnung der Skelettlinie von Tragflügel-Profilen,                */
/*  deren aerodynamischen Kenngrößen                                    */
/*  und deren Unterteilung in Segmente zur Benutzung mit XWing          */
/*                                                                      */
/************************************************************************/


/*

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef LINALGEBRA_H
#include "linalgebra.h"
#endif

#include <math.h>

void MatrixInverse(
        double  *A,
        int     n) {

  int   *indexc;                        // column
  int   *indexr;                        // row
  int   *ipiv;                          // pivot

  int   i, j, k;
  int   icol, irow;
  double big, bigger, dum, pivinv;

  indexc = new int[n];
  indexr = new int[n];
  ipiv = new int[n];

  for (j=0; j<n; j++) ipiv[j]=0;
  for (i=0; i<n; i++) {                 // main loop (columns)
    big=0.0;
    irow=0;
    icol=0;
    for (j=0; j<n; j++)                 // search for pivot
      if (ipiv[j]!=1)
        for (k=0; k<n; k++)
          if (ipiv[k]==0) {
            bigger=fabs(A[n*j+k]);
            if ( bigger>=big ) {
              big=bigger;
              irow=j;
              icol=k;
              };
            };
          // else
          //   if (ipiv[k]>1) 
          //     fatal_error(" MatrixInverse : singular matrix (pivot search)");
    ipiv[icol]++;
    if (irow!=icol)                     // interchange rows
      for (k=0; k<n; k++) {
        dum=A[n*irow+k];
        A[n*irow+k]=A[n*icol+k];
        A[n*icol+k]=dum;
        };
    indexr[i]=irow;
    indexc[i]=icol;
    // if (A[n*icol+irow]==0.0)
    //   fatal_error(" MatrixInverse : singular matrix (zero pivot)");
    pivinv=1.0/A[n*icol+icol];
    A[n*icol+icol]=1.0;
    for (k=0; k<n; k++)                 // divide row by pivot
      A[n*icol+k]*=pivinv;
    for (j=0; j<n; j++)                 // reduce rows
      if (j!=icol) {
        dum=A[n*j+icol];
        A[n*j+icol]=0.0;
        for (k=0; k<n; k++) {
          A[n*j+k]-=A[n*icol+k]*dum;
          };
        };
    };                                  // main loop (i) ends
    for (j=n; --j>=0; ) {               // undo column interchanges
      if (indexr[j]!=indexc[j])
        for (k=0; k<n; k++) {
          dum=A[n*k+indexr[j]];
          A[n*k+indexr[j]]=A[n*k+indexc[j]];
          A[n*k+indexc[j]]=dum;
          };
      };
  delete indexc;
  delete indexr;
  delete ipiv;
  return;
} /* MatrixInverse() */

void MatrixTranspose(
        double  *A,
        int     n) {

  int    i, k;
  double dum;

  for (i=0; i<n-1; i++)                 // Zeilen
    for (k=i+1; k<n; k++) {             // Spalten
      dum=A[n*i+k];
      A[n*i+k]=A[n*k+i];
      A[n*k+i]=dum;
      };

  return;
} /* MatrixTranspose() */

void MatrVecProd(
        double  *A,
        double  *b,
        double  *result,
        int     n) {

  int   i, k;
  
  for (i=0; i<n; i++) {
    result[i]=0.0;
    for (k=0; k<n; k++)
      result[i]+=A[n*i+k]*b[k];
    };

  return;
} /* MVprod() */
